bayeos.readChilds <- function(con=1){
	id=tail(.localenv$c[[con]]$path,1)[[1]]$id
	childs=bayeos.call(con,'TreeHandler.getChildren',id,'messung_massendaten',c('mitarbeiter','projekte'),.localenv$c[[con]]$activeFilter,
			.localenv$c[[con]]$missingInterval,.localenv$c[[con]]$timeFilter)
	childs=lapply(childs,function(x){ names(x)=.localenv$c[[con]]$treeNames; x})
	.localenv$c[[con]]$childNodes<-childs
}


bayeos.getChilds <- function(id=NA,isExtern=FALSE,maxDepth=-1,pathFilter='',classFilter='',con=1){
	if(.localenv$c[[con]]$bayeosVersion=='1.7'){
		warning('The server runs bayeos 1.7. This function is available for bayeos >= 1.8. Please upgrade the server to at least 1.8.')
		return(NULL);
	}
	
	if(is.na(id))
		id=tail(.localenv$c[[con]]$path,1)[[1]]$id
	pathFilter=sub('^\\.?/','',pathFilter) # strip ./ or / at beginning of pathFilter
	childs=bayeos.call(con,'TreeHandler.getAllChildren',as.integer(id),isExtern,c('mitarbeiter','projekte'),pathFilter,classFilter,as.integer(maxDepth),
			.localenv$c[[con]]$activeFilter,.localenv$c[[con]]$missingInterval,.localenv$c[[con]]$timeFilter)
	id=sapply(childs,function(x) x[[1]])
	name=sapply(childs,function(x) x[[8]])
	class=sapply(childs,function(x) x[[10]])
	path=sapply(childs,function(x) x[[9]])
	data.frame(id,name,class,path)
}


bayeos.find <- function(search,tree='Folder',con=1){
	tree=tolower(tree);
	if(is.na(classes[tree])){
		warning(paste('Unknown tree',tree))
		return(NULL)
	}	
	bayeos.getChilds(.localenv$c[[con]]$root_ids[classes[tree]],pathFilter=paste('**/',sub('/','\\\\/',search),sep=''),
			isExtern=(tree=='web'))
}

	
bayeos.ls <- function(con=1,full=FALSE){
	cur=tail(.localenv$c[[con]]$path,1)[[1]]
	cat(sapply(.localenv$c[[con]]$path,function(x) x$name),'\n',sep=' / ')
	#cat('Current Folder:',.localenv$c[[con]]$path[[cur]]$id,':\t',.localenv$c[[con]]$path[[cur]]$name,'\n\n')
	if(! is.null(cur$id_super))
		cat(cur$id_super,'\tparent_folder\t..\n')
	if(full)
		sapply(.localenv$c[[con]]$childNodes,function(x) print(x))
	else
		sapply(.localenv$c[[con]]$childNodes,function(x) cat(x$id,'\t',x$class,'\t',x$name,'\n'))
	if(length(.localenv$c[[con]]$childNodes)==0) df=data.frame(id=vector(),class=vector(),name=vector(),rec_start=vector(),rec_end=vector())
	else df=data.frame(
				id=sapply(.localenv$c[[con]]$childNodes,function(x) x$id),
				class=sapply(.localenv$c[[con]]$childNodes,function(x) x$class),
				name=sapply(.localenv$c[[con]]$childNodes,function(x) x$name),
				rec_start=as.POSIXct(sapply(.localenv$c[[con]]$childNodes,function(x) if(is.null(x$rec_start)) NA else x$rec_start),origin='1970-01-01',tz=.localenv$c[[con]]$tz),
				rec_end=as.POSIXct(sapply(.localenv$c[[con]]$childNodes,function(x) if(is.null(x$rec_end)) NA else x$rec_start),origin='1970-01-01',tz=.localenv$c[[con]]$tz)
		)
	invisible(df)
}

bayeos.cd <- function(id=NULL,con=1,ls=TRUE){
	if(is.null(id)){
		root=bayeos.call(con,'TreeHandler.getRoot','messung_ordner',.localenv$c[[con]]$activeFilter,.localenv$c[[con]]$missingInterval,.localenv$c[[con]]$timeFilter)
		names(root)=.localenv$c[[con]]$treeNames
		.localenv$c[[con]]$path<-list()
		.localenv$c[[con]]$path[[1]]<-root
	}
	cur=length(.localenv$c[[con]]$path)
	if(is.character(id) && id=='..'){
		if(cur==1) return(cat('Already at top level'))
		.localenv$c[[con]]$path[[cur]]<-NULL
		id=NULL
	}
	
	#checking for ID in current folder...
	if(is.character(id)){
		found=sapply(.localenv$c[[con]]$childNodes,function(x) x$name==id & x$class=='messung_ordner')
		if(is.list(found)) num_found=0
		else num_found=sum(found)
		if(num_found==0)
			return(cat('Folder',id,'not found in current folder'))  
		if(num_found>1)
			return(cat('Folder name ',id,' is not unique. Please use ID!'))
		id=.localenv$c[[con]]$childNodes[[(1:length(found))[found]]]$id
	}
	
	if(is.numeric(id)){
		found=sapply(.localenv$c[[con]]$childNodes,function(x) x$id==id)
		if(is.list(found)) num_found=0
		else num_found=sum(found)
		if(num_found==0){
			p=bayeos.getParents(id,con)
			if(is.null(p))
				return(cat('ID',id,'not found!'))
			.localenv$c[[con]]$path<-p
			
		} else 
			.localenv$c[[con]]$path[[cur+1]]<-.localenv$c[[con]]$childNodes[[(1:length(found))[found]]] 
	}
	bayeos.readChilds(con)
	if(ls) bayeos.ls(con)
}

bayeos.getParents <- function(id,con=1){
	parents=list()
	while(is.numeric(id)){
		o=bayeos.call(con,'ObjektHandler.getObjekt',as.integer(id),'messung_ordner')
		if(length(o)==1) return(NULL)
		id=o[[4]]
		parents=append(parents,list(list(id=o[[3]],id_super=o[[4]],class=o[[5]],name=o[[14]])),0)
	}
	parents
}

bayeos.createFolder <- function(name,con=1,ls=TRUE){
	cur=length(.localenv$c[[con]]$path)
	id_parent=.localenv$c[[con]]$path[[cur]]$id
	NewNode=bayeos.call(con,'TreeHandler.newNode','messung_ordner',name,id_parent)
	names(NewNode)=.localenv$c[[con]]$treeNames
	bayeos.readChilds(con)
	if(ls) bayeos.ls(con)
	NewNode$id
}

bayeos.updateSeries <- function(id,name=NA,tz=NA,
		description=NA,resolution=NA,planstart=NA,planend=NA,intervaltype=NA,
		con=1){
	attr=bayeos.call(con,'ObjektHandler.getObjekt',as.integer(id),'messung_massendaten')
	index=c('planstart'=16,'planend'=17,'name'=21,'description'=22,'resolution'=23,'intervaltype'=24,'tz'=25)
	func_env=environment()
	if(! is.na(tz)) tz=.localenv$c[[con]]$timezones[tz]
	if(! is.na(intervaltype)) intervaltype=.localenv$c[[con]]$intervaltypes[intervaltype]
	
	sapply(names(index),function(x){ if(is.na(get(x))) assign(x,attr[[index[x]]],envir=func_env) } )
	tz_string=names(.localenv$c[[con]]$timezones[.localenv$c[[con]]$timezones==tz])
	setBayeosConTZ(con,tz_string)
	if(is.character(planstart)){
		if(planstart=='') planstart=NULL
		else planstart=as.POSIXct(planstart,tz=tz_string)
	} 
	if(is.character(planend)){
		if(planend=='') planend=NULL
		else planend=as.POSIXct(planend,tz=tz_string)
	} 
#	return(list(name,description,as.integer(resolution),planstart,planend,intervaltype,tz))
	bayeos.call(con,'ObjektHandler.updateObjekt',as.integer(id),'messung_massendaten',
			list(name,description,as.integer(resolution),planstart,planend,as.integer(intervaltype),as.integer(tz)))
	
	
}


bayeos.createSeries <- function(name,con=1,ls=TRUE,warningLevel=1,tz='Etc/GMT-1',
		description='',resolution=600,planstart=NULL,planend=NULL,intervaltype='undefined'){
	setBayeosConTZ(con,tz)
	cur=length(.localenv$c[[con]]$path)
	id_parent=.localenv$c[[con]]$path[[cur]]$id
	NewNode=bayeos.call(con,'TreeHandler.newNode','messung_massendaten',name,id_parent)
	names(NewNode)=.localenv$c[[con]]$treeNames
	
	if(is.character(planstart)) planstart=as.POSIXct(planstart,tz=tz)
	if(is.character(planend)) planend=as.POSIXct(planend,tz=tz)
	bayeos.call(con,'ObjektHandler.updateObjekt',as.integer(NewNode$id),'messung_massendaten',
			list(name,description,as.integer(resolution),planstart,planend,
					as.integer(.localenv$c[[con]]$intervaltypes[intervaltype]),
					as.integer(.localenv$c[[con]]$timezones[tz])))
	bayeos.readChilds(con)
	if(ls) bayeos.ls(con)
	if(warningLevel>0){
		ref=bayeos.getReferences(NewNode$id,con)
		missing=checkReferences(ref,level=warningLevel)
		if(missing>0)	
			warning('The series does not have sufficient meta data. Please use GOAT or bayeos.createReferences to complement meta data!')
	}
	NewNode$id
}



bayeos.deleteNode <- function(id,con=1,confirm=TRUE,ls=TRUE,removeRows=FALSE){
	found=sapply(.localenv$c[[con]]$childNodes, function(x) x$id==id)
	if(all(! found) | is.list(found)){
		warning("Node not found in current folder")
		return(FALSE)
	}	
	node_index=(1:length(found))[found]
	node=.localenv$c[[con]]$childNodes[[node_index]]
	if(confirm){
		confirm=readline(prompt = paste('Do you really want to delete node ',node$name,'(type yes to confirm)'))
		if(confirm!='yes') return(FALSE)
	}
	if(node$class=='messung_massendaten' & removeRows)
		bayeos.call(con, 'MassenTableHandler.removeAllRows',as.integer(id))
	
	res=bayeos.call(con,'TreeHandler.deleteNode',as.integer(id))
	if(res) .localenv$c[[con]]$childNodes[[node_index]]<-NULL
	if(ls) bayeos.ls(con)
	res  
}
