\name{bayeos.cd}
\alias{bayeos.cd}
\alias{bayeos.ls}
\alias{bayeos.find}
\alias{bayeos.getChilds}
\alias{bayeos.createFolder}
\alias{bayeos.createSeries}
\alias{bayeos.updateSeries}
\alias{bayeos.deleteNode}
\title{Manage BayEOS Timeseries Tree}
\description{
Functions to list current folder, change folder and to create new folder or series in the current folder.
}
\usage{
bayeos.ls(con=1,full=FALSE)
bayeos.cd(id=NULL,con=1,ls=TRUE)
bayeos.createFolder(name,con=1,ls=TRUE)
bayeos.createSeries(name,con=1,ls=TRUE,warningLevel=1,tz='Etc/GMT-1',
			description='',resolution=600,planstart=NULL,planend=NULL,intervaltype='undefined')
bayeos.updateSeries(id,name=NA,tz=NA,description=NA,resolution=NA,planstart=NA,
			planend=NA,intervaltype=NA,con=1)
bayeos.deleteNode(id,con=1,confirm=TRUE,ls=TRUE,removeRows=FALSE)
bayeos.getChilds(id,isExtern=FALSE,maxDepth=-1,pathFilter='',classFilter='',con=1)
bayeos.find(search,tree='Folder',con=1)


}
%- maybe also 'usage' for other objects documented here.
\arguments{
	\item{con}{connection identifier returned by bayeos.connect}  
	\item{name}{name of the series or folder to create}
	\item{id}{numeric folder id or characters '..' for parent folder, NULL for root folder}  
	\item{ls}{print out modified directory}
	\item{full}{flag, if FALSE ls() returns short attribute columns} 
	\item{maxDepth}{interger, max. depth for tree search, -1==unlimited} 
	\item{pathFilter}{Filter string for ANT-like tree search. See below for details.} 
	\item{search}{String to search nodes. For substring search use '*temp*'}
	\item{classFilter}{Object type filter string}
	\item{tree}{Tree to search in. One of 'folder','device','compartment','location','target','unit','web'}
	\item{planstart}{Planned start of a time series. Date time string interpreted 
		in the local time zone of the series.}
	\item{planend}{Planned end of a time series. Date time string interpreted 
		in the local time zone of the series.}
	 
}

\value{
  bayeos.ls prints out the current directory and returns an invisible data.frame with columns id,class,name,rec_start,rec_end.
  bayeos.createFolder and bayeos.createSeries return ID of new node.
  bayeos.getChilds and bayeos.find returns a data frame with the columns id,class,name,path
}

\author{S. Holzheu, O. Archner}


\examples{
 	
	\dontrun{
	# open connection 
	bayeos.connect('http://www.bayceer.uni-bayreuth.de/servlet/XMLServlet','gast','gast')

	# show the current folder
	bayeos.ls()
	
	# change to a subfolder
	bayeos.cd(14333)
	# change to a folder 
	bayeos.cd(36076)
	# change to the parent folder
	bayeos.cd('..')
	# change to root folder
	bayeos.cd('root')

	## Childs
	# get all childs of a specified node
	bayeos.getChilds(83)
	# get all childs of a specified node using filter
	# ** is a wildcard for any character int the path including the folder separater
	# * is a wildcard for all characters except the folder separator
	# To find all Nodes beginning with "temparature" anywhere in the tree  
	bayeos.getChilds(83,pathFilter='**/temperature*')

	## Searching the trees
	# Seraching all nodes having temperatur somewhere in the name
	bayeos.find('*temperatur*')
	# Finding Unit 'g/l'
	bayeos.find('m/s',tree='Unit')
	# getting all units 
	bayeos.find('*',tree='Unit')
	
	
	# create folder and series (only works, when rights are sufficient)
	newFolder=bayeos.createFolder('test folder')
	bayeos.createReferences(newFolder,'Somewhere','Location')
	newNode=bayeos.createSeries('test series')
	bayeos.createReferences(newNode,'Celsius','Unit')
	bayeos.deleteNode(newNode)
	# close connection
	bayeos.close()
	}

}
\keyword{data}
\keyword{timeseries}
